@@ -8,12 +8,16 @@
 #include "input/togglablegesture.h"
 #include "modules/virtual-output/virtualoutputmanager.h"
 #include "modules/window-management/windowmanagement.h"
+#include "utils/fpsdisplaymanager.h"
 
 #include <wglobal.h>
 #include <wqmlcreator.h>
 #include <wseat.h>
 #include <wxdgdecorationmanager.h>
 #include <wextforeigntoplevellistv1.h>
+#include <woutputmanagerv1.h>
+
+#include <xcb/xproto.h>
 
 Q_MOC_INCLUDE(<wtoplevelsurface.h>)
 Q_MOC_INCLUDE(<wxdgsurface.h>)
@@ -53,6 +57,8 @@ class WForeignToplevel;
 class WExtForeignToplevelListV1;
 class WOutputManagerV1;
 class WLayerSurface;
+class WSessionLockManager;
+class WSessionLock;
 WAYLIB_SERVER_END_NAMESPACE
 
 QW_BEGIN_NAMESPACE
@@ -93,6 +99,7 @@ class ILockScreen;
 class UserModel;
 class DDMInterfaceV1;
 class TreelandConfig;
+class FpsDisplayManager;
 struct wlr_idle_inhibitor_v1;
 struct wlr_output_power_v1_set_mode_event;
 struct wlr_ext_foreign_toplevel_image_capture_source_manager_v1_request;
@@ -101,6 +108,14 @@ QW_BEGIN_NAMESPACE
 class qw_ext_foreign_toplevel_image_capture_source_manager_v1;
 QW_END_NAMESPACE
 
+struct Session {
+    uid_t uid = 0;
+    WSocket *socket = nullptr;
+    WXWayland *xwayland = nullptr;
+    quint32 noTitlebarAtom = XCB_ATOM_NONE;
+    SurfaceWrapper *lastActivatedSurface = nullptr;
+};
+
 class Helper : public WSeatEventFilter
 {
     friend class RootSurfaceContainer;
@@ -115,6 +130,7 @@ class Helper : public WSeatEventFilter
     Q_PROPERTY(Workspace* workspace READ workspace CONSTANT FINAL)
     Q_PROPERTY(TreelandConfig* config READ config CONSTANT FINAL)
     Q_PROPERTY(bool blockActivateSurface READ blockActivateSurface WRITE setBlockActivateSurface NOTIFY blockActivateSurfaceChanged FINAL)
+    Q_PROPERTY(bool noAnimation READ noAnimation WRITE setNoAnimation NOTIFY noAnimationChanged FINAL)
     QML_ELEMENT
     QML_SINGLETON
 
@@ -172,8 +188,9 @@ public:
     Q_INVOKABLE void addOutput();
 
     void addSocket(WSocket *socket);
-    WXWayland *createXWayland();
     void removeXWayland(WXWayland *xwayland);
+    WXWayland *xwaylandForUid(uid_t uid, bool createIfMissing = true);
+    WSocket *waylandSocketForUid(uid_t uid, bool createIfMissing = true);
 
     WSocket *defaultWaylandSocket() const;
     WXWayland *defaultXWaylandSocket() const;
@@ -216,12 +233,15 @@ public:
 
     void setBlockActivateSurface(bool block);
     bool blockActivateSurface() const;
+    bool noAnimation() const;
+    void toggleFpsDisplay();
+
 public Q_SLOTS:
     void activateSurface(SurfaceWrapper *wrapper, Qt::FocusReason reason = Qt::OtherFocusReason);
     void forceActivateSurface(SurfaceWrapper *wrapper,
                               Qt::FocusReason reason = Qt::OtherFocusReason);
     void fakePressSurfaceBottomRightToReszie(SurfaceWrapper *surface);
-    bool surfaceBelongsToCurrentUser(SurfaceWrapper *wrapper);
+    bool surfaceBelongsToCurrentSession(SurfaceWrapper *wrapper);
 
 Q_SIGNALS:
     void socketEnabledChanged();
@@ -233,8 +253,10 @@ Q_SIGNALS:
     void outputModeChanged();
 
     void currentModeChanged();
+    void noAnimationChanged();
 
     void blockActivateSurfaceChanged();
+    void requestQuit();
 
 private Q_SLOTS:
     void onShowDesktop();
@@ -267,10 +289,10 @@ private:
     void onSessionLock();
     void onSessionUnlock();
     void handleNewForeignToplevelCaptureRequest(wlr_ext_foreign_toplevel_image_capture_source_manager_v1_request *request);
+    void onExtSessionLock(WSessionLock *lock);
 
 private:
     void allowNonDrmOutputAutoChangeMode(WOutput *output);
-
     int indexOfOutput(WOutput *output) const;
 
     SurfaceWrapper *keyboardFocusSurface() const;
@@ -301,15 +323,29 @@ private:
     void setWorkspaceVisible(bool visible);
     void restoreFromShowDesktop(SurfaceWrapper *activeSurface = nullptr);
     void updateIdleInhibitor();
+    void setNoAnimation(bool noAnimation);
+
+    std::shared_ptr<Session> sessionForUid(uid_t uid) const;
+    std::shared_ptr<Session> sessionForXWayland(WXWayland *xwayland) const;
+    std::shared_ptr<Session> sessionForSocket(WSocket *socket) const;
+    std::shared_ptr<Session> ensureSession(uid_t uid);
+    void updateActiveUserSession(uid_t uid);
+    bool isXWaylandClient(WClient *client);
 
     static Helper *m_instance;
     TreelandConfig *m_config = nullptr;
+    FpsDisplayManager *m_fpsManager = nullptr;
 
     CurrentMode m_currentMode{ CurrentMode::Normal };
 
+    // Sessions
+    std::weak_ptr<Session> m_activeSession;
+    QList<std::shared_ptr<Session>> m_sessions;
+
     // qtquick helper
     WOutputRenderWindow *m_renderWindow = nullptr;
     QQuickItem *m_dockPreview = nullptr;
+    QQuickItem *m_fpsDisplay = nullptr;
 
     // gesture
     WServer *m_server = nullptr;
@@ -318,7 +354,6 @@ private:
     TogglableGesture *m_windowGesture = nullptr;
 
     // wayland helper
-    WSocket *m_socket = nullptr;
     WSeat *m_seat = nullptr;
     WBackend *m_backend = nullptr;
     qw_renderer *m_renderer = nullptr;
@@ -331,7 +366,6 @@ private:
     qw_output_power_manager_v1 *m_outputPowerManager = nullptr;
     qw_ext_foreign_toplevel_image_capture_source_manager_v1 *m_foreignToplevelImageCaptureManager = nullptr;
     ShellHandler *m_shellHandler = nullptr;
-    WXWayland *m_defaultXWayland = nullptr;
     WXdgDecorationManager *m_xdgDecorationManager = nullptr;
     WForeignToplevel *m_foreignToplevel = nullptr;
     WExtForeignToplevelListV1 *m_extForeignToplevelListV1 = nullptr;
@@ -346,7 +380,10 @@ private:
     VirtualOutputV1 *m_virtualOutput = nullptr;
     PrimaryOutputV1 *m_primaryOutputV1 = nullptr;
     DDMInterfaceV1 *m_ddmInterfaceV1 = nullptr;
-
+#ifdef EXT_SESSION_LOCK_V1
+    WSessionLockManager *m_sessionLockManager = nullptr;
+    QTimer *m_lockScreenGraceTimer = nullptr;
+#endif
     // private data
     QList<Output *> m_outputList;
     QPointer<QQuickItem> m_taskSwitch;
@@ -370,7 +407,17 @@ private:
     IMultitaskView *m_multitaskView{ nullptr };
     UserModel *m_userModel{ nullptr };
 
-    quint32 m_atomDeepinNoTitlebar;
-
     bool m_blockActivateSurface{ false };
+
+    bool m_noAnimation{ false };
+
+    struct PendingOutputConfig {
+        qw_output_configuration_v1 *config = nullptr;
+        QList<WOutputState> states;
+        int pendingCommits = 0;
+        bool allSuccess = true;
+    };
+    PendingOutputConfig m_pendingOutputConfig;
+
+    void onOutputCommitFinished(qw_output_configuration_v1 *config, bool success);
 };
